% Matlab Mistuning Code: Version 1.0
%  
% Created by: Benjamin Shapiro, 
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
% 
% *********************************************************
% 
% File: tuneigen.m 
% Purpose: Computes tuned eigenvalues and eigenvectors. 
% Last Update: 25/03/98 by B.Shapiro
%
% *********************************************************

fprintf('Computing tuned eigenvalues/vectors ... \n')

  % Clear space for data structures:
  UTun = zeros(d*r,d*r);   % Tuned left eigenvectors.
  VTun = zeros(d*r,d*r);   % Tuned right eigenvectors.
  Q = zeros(d,d,r);        % Qj matrices.
  rev = zeros(d,d,r);      % right (block) eigenvectors
  lev = zeros(d,d,r);      % left (block) eigenvectors
  pblock = zeros(d,r*d);   % temp variable
  EigenTun = zeros(1,r*d); % List of eigenvalues.
  prow = zeros(1,r);       % temp variable.
  % Largest first to optimize memory.


% Construct Qj matrices, find tuned eigenvalues/vectors:
for j = 1:r
  % Find Qj:
  % Qj = M1 + pj M2 + pj^2 M3 + ... + pj^(r-1) Mr
    prow = p(j,r).^(0:1:r-1);
    pblock = kron(prow, eye(d) );
    Q(:,:,j) = MrowTun*pblock.';  % .' is non-conjugate transpose.
  
     % NEW: Output from eig2:
       X = eig2( Q(:,:,j) );

  % The eigenvalues:
    % OLD: EigenTun((j-1)*d+1: j*d) = eig( Q(:,:,j) );
    EigenTun((j-1)*d+1: j*d) = X(:,1); % NEW!

  % Right eigenvectors (blocks)
    % OLD: [rights, LambdaR] = eig( Q(:,:,j) );
      rights = X(:,2:3); % NEW!
      rev(:,:,j) = rights;
  % Right eigenvectors (full)
    VTun(:,(j-1)*d+1:j*d) = (pblock.')*rights / sqrt(r);
    % Normalized to ||VTun(:,l)|| = 1.

  % Left eigenvectors (blocks)
    % I could compute these by evaluating 
    % [lefts, LambdaL] = eig( Q(:,:,j)' ); lev(:,:,j) = lefts;
    % but Matlab screws up the order so jth lev
    % does not match jth rev! So compute by inverse---faster!
    % OLD: lev(:,:,j) = inv( rev(:,:,j) );
      lev(:,:,j) = X(:,4:5);

    % Check to see if ill-conditioned eigenvectors:
      illcond = 1e5;
      if norm( lev(:,:,j) ) > illcond 
        fprintf('\t Warning: mode %g eigenvectors illconditioned. \n',j)
        fprintf('\t \t (I will fix this later.) \n')
        % Note: each mode j contains d left/right eigenvect.
      end

  % Left eigenvectors (full)
    UTun((j-1)*d+1:j*d,:) = lev(:,:,j)*conj( pblock ) / sqrt(r);
    % Normalized so VTun*UTun = I.      
end




fprintf('\t \t \t \t \t \t done! \n')

% Plot tuned eigenvalues:
xmin = 1.1 * min( real( EigenTun ) );
xmax = max( 0, max( real( EigenTun ) ) );
ymax = max( imag( EigenTun ) );
ymin = min( abs( imag( EigenTun ) ) );
delta = ymax - ymin;
ymax = ymax + delta/10;
ymin = ymin - delta/10;
  plot( EigenTun, 'r*')
  title('Tuned Eigenvalues: numbered by nodal diameter.');
  axis([xmin xmax ymin ymax]);
  xlabel('real');
  ylabel('imag');
  addinfo('',date,'Complex conjugates not shown.','B. Shapiro, Caltech');
  % Number eigenvalues by nodal diameter
    for j = 1:r
      for k = 1:d
        x = real( EigenTun( (j-1)*d + k) );
        y = imag( EigenTun( (j-1)*d + k) );
        y = y + delta/40;
        h = text(x,y,num2str(j));
        set(h,'horizontalalignment','center')
      end
    end
clear xmin xmax ymin ymax delta x y h j k;
% End plot tuned eigenvalues.
% Print plot to file:
  print -depsc Outputs/TunEigPlot.eps
  fprintf('Tuned eigenvalue plot saved to: Outputs/TunEigPlot.eps \n')



% DESCRIPTION OF DATA STRUCTURES:
%   1) Q: Qj = M1 + pj M2 + pj^2 M3 + .. + pj^(r-1) Mr
%         at z = 0. Eigenvalues Qj's = eigenvalues of M(0).
%      Size: r of d by d matrices.
%      Syntax: Q(a,b,j) is a (row) b (column) element of Qj.
%
%   2) VTun: Matrix of right (column) eigenvectors of M(0).
%            Column (j-1)d+i gives ith r.e.v for jth nodal diam.
%            Normalized so each column Vj has ||Vj|| = 1.
%      Size: rd by rd matrix.
%      Syntax: UTun(i,j) denotes ith row, jth column.
%
%   3) UTun: Matrix of left (row) eigenvectors of M(0).
%            Row (j-1)d+i gives ith l.e.v for jth nodal diam.
%            Inverse of VTun: VTun*UTun = UTun*VTun = I.
%            (Unless some l.e.v's ill-conditioned => zeroed.)
%      Size: rd by rd matrix.
%      Syntax: UTun(i,j) denotes ith row, jth column.
%
%   3) EigenTun: List of tuned eigenvalues of M(0). Element
%            (j-1)d+i gives ith e.v. for jth nodal diam.
%      Size: rd vector.
%      Syntax: EigenTun(i) denotes ith tuned eigenvalue.
%






% TO DO: 25/03/98.
%   Fix ill-cond: first cut, zero out (only) errant lev's.
%   This is a very rare problem, it can wait.
