% Matlab Mistuning Code: Version 2.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
%
% *********************************************************
%
% File: show_modal.m
% Purpose: Given a mistuning vector z, this file plots
%   (using an EXACT solution of the model at z) the
%     1) mistuning z 
%     2) tuned vs mistuned eigenvalues
%     3) tuned vs mistuned response at `worst' frequency.
%     4) modal decomposition of tuned/mistuned resp. in 3).
%   This file loads variables from Mistuning.m.
%   (No need to run Mistuning if these saved files exist.)
%   Aim: test if mistuning can make it seem as if there is
%     an erraneous nodal diameter forcing.
% Last Update: 06/12/98 by B.Shapiro
%
% *********************************************************

% Load saved data:
  load('Data/modelvar');
  load('Outputs/TunEig');
  load('Outputs/forced');
  load('Outputs/PertEig');
% End load.




% Choose mistuning size:
  sz = 0.01;

  % Pick a random mistuning y:
    randn('state',sum(100*clock)); % new seed.
    y = sz * randn(1,r);
    rnd = 1;
 
  % or Pick a modal mistuning y:
  % md = 2;
  % y = sz * cos( (2*pi*md/r) * [1:1:r] );
  % rnd = 0;

  % Allowable combinatoric pattern:
    % s = randperm(15);
    % q = randperm(30);
    % Z = arrange(s,q);
    % y = Z(1,:);
  % Using arrange.m originally from PWsearch/Source




% Print output:
  fprintf('\n');
  if rnd
    fprintf('Showing (exact) output for: \n');
    fprintf('\t random mistuning of size %g,\n',sz);
    fprintf('\t forcing in spatial mode %g. \n',l);
    fprintf('Plot saved to: Outputs/show_modal.eps. \n');
    fprintf('\n'); 
  else
    fprintf('Showing (exact) output for: \n');
    fprintf('\t modal mistuning of size %g, mode %g,\n',sz,md);
    fprintf('\t forcing in spatial mode %g. \n',l);
    fprintf('Plot saved to: Outputs/show.eps. \n');
    fprintf('\n'); 
  end
% End print.



% Initialize plots:
  figure(3); 
  clf;



% Top left, mistuning z:
  subplot(2,2,1);
    plot(y,'bo-')
    title('Chosen mistuning pattern');
    xlabel('slot number');
    ylabel('mistuning');
      ymax = max(y);
      ymin = min(y);
      delta = max(ymax - ymin,1e-3);  
      ymax = ymax + delta/20;
      ymin = ymin - delta/20;
    axis([0 r+1 ymin ymax]);
% End top left.




% Top right, tuned vs mistuned eigenvalues:
  subplot(2,2,2);

  % Compute mistuned eigenvalues:
    % Construct M(y),B(y).
    clear Mofy;                    
      Mofy = zeros(r*d,r*d);  % Initialize   
      Bofy = zeros(r*d,1);    % Initialize  
    Mofy(1:d,:) = double( subs(Mrow,z,y) );
    Bofy(1:d,1) = double( subs(beta,z,y) );
    for j = 1:r-1
       Mofy(j*d+1:(j+1)*d,:) = ...
                double( rot( subs( Mrow, z, rot(y,j,1,r) )  ,r-j,d,r) );
       Bofy(j*d+1:(j+1)*d,1) = ...
                p(l*j,r)* double( subs(beta,z,rot(y,j,1,r)) );
    end
    % Got M(y),B(y).

    EigenMist = eig( Mofy );
  % Got mistuned eigenvalues.

  % Do the plot:
    xmin = 1.15 * min( real( EigenMist ) );
    xmax = max( 0, max( real( EigenMist ) ) );
    ymax = max( imag( EigenMist ) );
    ymin = min( abs( imag( EigenMist ) ) );
    delta = ymax - ymin;
    ymax = ymax + delta/6;
    ymin = ymin - delta/6;
    plot( real(EigenTun), imag(EigenTun), 'r.', ...   % tuned eigenvalues
          real(EigenMist), imag(EigenMist), 'b+', ... % mistuned eigenvalues
            real(EigenTun( (l-1)*d+1:l*d )), ...      % the (tuned) forced mode.
            imag(EigenTun( (l-1)*d+1:l*d )), 'go' )
    title('Tuned vs Mistuned Eigenvalues');
    axis([xmin xmax ymin ymax]);
    xlabel('real');
    ylabel('imag');
% End top right.




% Bottom left, tuned vs mistuned response at worst case freq:
  subplot(2,2,3);

  % Now I need to know the (approximate) worst case freq:  
    [damping, numb] = max(real(EigenMist));
    worst_w = abs( imag(EigenMist(numb)) ); % It may chosen complex-conj.
  % Got worst case (approximately).


  % Define X(0,worst_w) and X(y,worst_w)
    clear X0 Xy
    X0_1 = XTun(l,worst_w,EigenTun,Psi0,d);  % Tuned response of blade 1.
    X0 = X0_1(1)*p(l,r).^(0:r-1);            % Generate all blades by sym.
    Xy = exact_resp(Mofy,Bofy,worst_w,r,d);  % again, r "position" entries.
    mag = max(abs(Xy))/abs(X0(1));  % Again, X0(1) takes position.
    amp = ['Amp ' num2str(mag)];
  % Got X0 and Xy.


  % Do the plot:
    plot(real(X0), imag(X0), 'r.', real(Xy), imag(Xy),'bs'); 
    titlestr = ['Response at freq. w = ' num2str(worst_w)];
    title( titlestr );
    xlabel('real');
    ylabel('imag');
    xmax = 1.6*max(abs(Xy));
    axis([-xmax xmax -xmax xmax]);
    axis equal;
% End bottom left






% Bottom right, modal decomposition of tuned and mistuned response:
  subplot(2,2,4);
    % Note: Xy is column vector, X0 is row vector.
    modal_tun = zeros(r,1);
    modal_mist = zeros(r,1);
    for j = 1:r
      modal_tun(j) =   abs( ( p(-j,r).^(0:1:r-1) )*X0.' );
      modal_mist(j) =  abs( ( p(-j,r).^(0:1:r-1) )*Xy   );
      % Basically computing: aj = |<X,Pj>| = |X.Conj[Pj]|,
      % the inner-product gives modal content aj. I do -j
      % in the p(-j) to get the Conj[Pj]. X0.' is so that
      % it takes regular dot product---not the conjugate.
    end

  % Do the plot:
    bar(modal_tun,'r')
    hold on
    bar(modal_mist,0.45,'b')
    title('Modal Decomposition');
    xlabel('nodal diameter');
    ylabel('modal content');
    xmin = 0;
    xmax = r+1;
    ymin = 0;
    ymax = 1.05 * max( [modal_tun; modal_mist ] );  
    axis([xmin xmax ymin ymax]);
% End bottom right



% Comments for entire plot:
  addinfo(amp, date, 'Exact: red = tuned, blue = mistuned, green = forced.', ...
          'MAST: B. Shapiro, Caltech');
 
% Print plot to file:
  print -depsc Outputs/show_modal.eps





% Aside: This piece of code automatically displays the first element 
%   for the state of each blade (the "position", not the "velocity").
%   For the exact mistuned response, exact_resp is wired this way also.
%   If I want to change to looking at velocity I need to change two
%   pieces of code (this one and exact_resp).    B. Shapiro, 05/01/98
